use celery::beat::DeltaSchedule;
use celery::broker::AMQPBroker;
use lazy_static::lazy_static;
use std::sync::{Arc, Mutex};
use std::thread;
use std::time;
use stdext::duration::DurationExt;

use crate::config;

pub mod update_info;

pub const QUEUE_NAME: &str = "celery";

lazy_static! {
    pub static ref APP: Mutex<Option<Arc<celery::Celery<AMQPBroker>>>> = Mutex::new(None);
}

#[tokio::main]
pub async fn init() {
    *APP.lock().unwrap() = Some(
        celery::app!(
            broker = AMQPBroker { &config::CONFIG.amqp_url },
            tasks = [
                update_info::update_info,
            ],
            task_routes = [
                "*" => QUEUE_NAME,
            ],
            prefetch_count = 2,
            heartbeat = Some(10)
        )
        .await
        .unwrap(),
    );
}

pub fn init_blocking() {
    thread::spawn(|| {
        tokio::runtime::Runtime::new().unwrap().spawn_blocking(init);
    });

    let dur = time::Duration::from_secs(1);
    let mut i = 0;
    loop {
        if APP.lock().unwrap().is_some() {
            break;
        }
        thread::sleep(dur);
        i += 1;

        if i >= 10 {
            panic!("Worker not initialized after 10 seconds!");
        }
    }
}

pub fn beat() -> impl std::future::Future<
    Output = Result<
        celery::beat::Beat<AMQPBroker, celery::beat::LocalSchedulerBackend>,
        celery::error::BeatError,
    >,
> {
    celery::beat!(
        broker = AMQPBroker { &config::CONFIG.amqp_url },
        tasks = [
            "update_info" => {
                update_info::update_info,
                schedule = DeltaSchedule::new(time::Duration::from_minutes(5)),
                args = (),
            }
        ],
        task_routes = [
            "*" => QUEUE_NAME,
        ]
    )
}
